@@ -34,8 +34,12 @@
 #include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-list-view.h"
 #include "file-manager/fm-tree-view.h"
+#if ENABLE_EMPTY_VIEW
+#include "file-manager/fm-empty-view.h"
+#endif /* ENABLE_EMPTY_VIEW */
 #include "nautilus-information-panel.h"
 #include "nautilus-history-sidebar.h"
+#include "nautilus-places-sidebar.h"
 #include "nautilus-notes-viewer.h"
 #include "nautilus-emblem-sidebar.h"
 #include "nautilus-image-properties-page.h"
@@ -49,21 +53,24 @@
 #include "nautilus-navigation-window.h"
 #include "nautilus-shell-interface.h"
 #include "nautilus-shell.h"
+#include "nautilus-window-bookmarks.h"
 #include "nautilus-window-private.h"
+#include "nautilus-window-manage-views.h"
+#include <libxml/xmlsave.h>
+#include <glib/gstdio.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
+#include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string-list.h>
 #include <eel/eel-string.h>
-#include <eel/eel-vfs-extensions.h>
-#include <eel/eel-gtk-extensions.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkinvisible.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwindow.h>
 #include <libgnome/gnome-config.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-authentication-manager.h>
 #include <libgnomeui/gnome-client.h>
@@ -73,18 +80,22 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-metafile-factory.h>
 #include <libnautilus-private/nautilus-module.h>
-#include <libnautilus-private/nautilus-sound.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-private/nautilus-desktop-link-monitor.h>
 #include <libnautilus-private/nautilus-directory-private.h>
+#include <libnautilus-private/nautilus-signaller.h>
+#include <libnautilus-extension/nautilus-menu-provider.h>
 #include <bonobo-activation/bonobo-activation.h>
+#ifdef HAVE_STARTUP_NOTIFICATION
 #define SN_API_NOT_YET_FROZEN Yes_i_know_DO_IT
 #include <libsn/sn-launchee.h>
+#endif
 
 /* Needed for the is_kdesktop_present check */
 #include <gdk/gdkx.h>
@@ -116,6 +127,8 @@ static void     update_session                    (gpointer                  cal
 static void     init_session                      (void);
 static gboolean is_kdesktop_present               (void);
 
+static char    *save_session_to_file              (void);
+
 BONOBO_CLASS_BOILERPLATE (NautilusApplication, nautilus_application,
 			  BonoboGenericFactory, BONOBO_TYPE_GENERIC_FACTORY)
 
@@ -151,6 +164,12 @@ nautilus_application_get_spatial_window_list (void)
 	return nautilus_application_spatial_window_list;
 }
 
+unsigned int
+nautilus_application_get_n_windows (void)
+{
+	return g_list_length (nautilus_application_window_list) +
+	       g_list_length (nautilus_application_desktop_windows);
+}
 
 static void
 nautilus_application_instance_init (NautilusApplication *application)
@@ -158,6 +177,8 @@ nautilus_application_instance_init (NautilusApplication *application)
 	/* Create an undo manager */
 	application->undo_manager = nautilus_undo_manager_new ();
 
+	application->shell = nautilus_shell_new (application);
+	
 	/* Watch for volume mounts so we can restore open windows
 	 * This used to be for showing new window on mount, but is not
 	 * used anymore */
@@ -174,8 +195,12 @@ nautilus_application_instance_init (NautilusApplication *application)
 	fm_icon_view_register ();
 	fm_desktop_icon_view_register ();
 	fm_list_view_register ();
+#if ENABLE_EMPTY_VIEW
+	fm_empty_view_register ();
+#endif /* ENABLE_EMPTY_VIEW */
 
 	/* register sidebars */
+	nautilus_places_sidebar_register ();
 	nautilus_information_panel_register ();
 	fm_tree_view_register ();
 	nautilus_history_sidebar_register ();
@@ -211,6 +236,11 @@ nautilus_application_destroy (BonoboObject *object)
 	
 	g_object_unref (application->undo_manager);
 
+	if (application->shell_registered) {
+		bonobo_activation_unregister_active_server (SHELL_IID, BONOBO_OBJREF (application->shell));
+	}
+	bonobo_object_unref (application->shell);
+
 	EEL_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
 }
 
@@ -223,7 +253,6 @@ check_required_directories (NautilusApplication *application)
 	char *directories_as_string;
 	char *error_string;
 	char *detail_string;
-	char *dialog_title;
 	GtkDialog *dialog;
 	int failed_count;
 	
@@ -250,20 +279,18 @@ check_required_directories (NautilusApplication *application)
 		directories_as_string = eel_string_list_as_string (directories, ", ", EEL_STRING_LIST_ALL_STRINGS);
 
 		if (failed_count == 1) {
-			dialog_title = _("Couldn't Create Required Folder");
 			error_string = g_strdup_printf (_("Nautilus could not create the required folder \"%s\"."),
 							directories_as_string);
 			detail_string = _("Before running Nautilus, please create the following folder, or "
 					  "set permissions such that Nautilus can create it.");
 		} else {
-			dialog_title = _("Couldn't Create Required Folders");
 			error_string = g_strdup_printf (_("Nautilus could not create the following required folders: "
 							  "%s."), directories_as_string);
   			detail_string = _("Before running Nautilus, please create these folders, or "
 					  "set permissions such that Nautilus can create them.");
 		}
 		
-		dialog = eel_show_error_dialog (error_string, detail_string, dialog_title, NULL);
+		dialog = eel_show_error_dialog (error_string, detail_string, NULL);
 		/* We need the main event loop so the user has a chance to see the dialog. */
 		nautilus_main_event_loop_register (GTK_OBJECT (dialog));
 
@@ -276,15 +303,6 @@ check_required_directories (NautilusApplication *application)
 	return failed_count == 0;
 }
 
-static int
-nautilus_strv_length (const char * const *strv)
-{
-	const char * const *p;
-
-	for (p = strv; *p != NULL; p++) { }
-	return p - strv;
-}
-
 static Nautilus_URIList *
 nautilus_make_uri_list_from_shell_strv (const char * const *strv)
 {
@@ -292,14 +310,14 @@ nautilus_make_uri_list_from_shell_strv (const char * const *strv)
 	Nautilus_URIList *uri_list;
 	char *translated_uri;
 
-	length = nautilus_strv_length (strv);
+	length = g_strv_length ((char **) strv);
 
 	uri_list = Nautilus_URIList__alloc ();
 	uri_list->_maximum = length;
 	uri_list->_length = length;
 	uri_list->_buffer = CORBA_sequence_Nautilus_URI_allocbuf (length);
 	for (i = 0; i < length; i++) {
-		translated_uri = eel_make_uri_from_shell_arg (strv[i]);
+		translated_uri = gnome_vfs_make_uri_from_shell_arg (strv[i]);
 		uri_list->_buffer[i] = CORBA_string_dup (translated_uri);
 		g_free (translated_uri);
 		translated_uri = NULL;
@@ -346,13 +364,41 @@ migrate_old_nautilus_files (void)
 		eel_show_info_dialog (_("A link called \"Link To Old Desktop\" has been created on the desktop."),
 				      _("The location of the desktop directory has changed in GNOME 2.4. "
 					"You can open the link and move over the files you want, then delete the link."),
-				      _("Migrated Old Desktop"),
 				      NULL);
 	}
 	g_free (old_desktop_dir);
 	g_free (migrated_file);
 }
 
+static void
+menu_provider_items_updated_handler (NautilusMenuProvider *provider, GtkWidget* parent_window, gpointer data)
+{
+
+	g_signal_emit_by_name (nautilus_signaller_get_current (),
+			       "popup_menu_changed");
+}
+
+static void
+menu_provider_init_callback (void)
+{
+        GList *items;
+        GList *providers;
+        GList *l;
+
+        providers = nautilus_module_get_extensions_for_type (NAUTILUS_TYPE_MENU_PROVIDER);
+        items = NULL;
+
+        for (l = providers; l != NULL; l = l->next) {
+                NautilusMenuProvider *provider = NAUTILUS_MENU_PROVIDER (l->data);
+
+		g_signal_connect_after (G_OBJECT (provider), "items_updated",
+                           (GCallback)menu_provider_items_updated_handler,
+                           NULL);
+        }
+
+        nautilus_module_extension_list_free (providers);
+}
+
 static void
 finish_startup (NautilusApplication *application)
 {
@@ -360,10 +406,10 @@ finish_startup (NautilusApplication *application)
 	nautilus_module_init ();
 
 	nautilus_module_add_type (FM_TYPE_DITEM_PAGE);
-	
-	/* initialize the sound machinery */
-	nautilus_sound_init ();
 
+	/* attach menu-provider module callback */
+	menu_provider_init_callback ();
+	
 	/* initialize URI authentication manager */
 	gnome_authentication_manager_init ();
 
@@ -456,6 +502,7 @@ nautilus_application_startup (NautilusApplication *application,
 			      gboolean browser_window,
 			      const char *startup_id,
 			      const char *geometry,
+			      const char *session_to_load,
 			      const char *urls[])
 {
 	CORBA_Environment ev;
@@ -485,12 +532,16 @@ nautilus_application_startup (NautilusApplication *application,
 	while (TRUE) {
 		/* Try to register the file manager view factory. */
 		result = nautilus_bonobo_activation_register_for_display
-			(FACTORY_IID, BONOBO_OBJREF (application));
+			(SHELL_IID, BONOBO_OBJREF (application->shell));
 
 		switch (result) {
 		case Bonobo_ACTIVATION_REG_SUCCESS:
 			/* We are registered and all is right with the world. */
+			application->shell_registered = TRUE;
 			finish_startup (application);
+			message = NULL;
+			detailed_message = NULL;
+			break;
 		case Bonobo_ACTIVATION_REG_ALREADY_ACTIVE:
 			/* Another copy of nautilus already is running and registered. */
 			message = NULL;
@@ -551,7 +602,7 @@ nautilus_application_startup (NautilusApplication *application,
 
 		/* Get the shell object. */
 		if (message == NULL) {
-			shell = bonobo_activation_activate_from_id (SHELL_IID, 0, NULL, NULL);
+			shell = bonobo_activation_activate_from_id (SHELL_IID, Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, NULL, NULL);
 			if (!CORBA_Object_is_nil (shell, &ev)) {
 				break;
 			}
@@ -563,7 +614,7 @@ nautilus_application_startup (NautilusApplication *application,
 				/* FIXME bugzilla.gnome.org 42538: When can this happen? */
 				message = _("Nautilus can't be used now, due to an unexpected error.");
 				detailed_message = _("Nautilus can't be used now, due to an unexpected error "
-						     "from Bonobo when attempting to locate the factory."
+						     "from Bonobo when attempting to locate the factory. "
 						     "Killing bonobo-activation-server and restarting Nautilus may help fix the problem.");
 			} else {
 				num_failures++;
@@ -578,7 +629,7 @@ nautilus_application_startup (NautilusApplication *application,
 		}
 
 		if (message != NULL) {
-			dialog = eel_show_error_dialog_with_details (message, NULL, NULL, detailed_message, NULL);
+			dialog = eel_show_error_dialog_with_details (message, NULL, detailed_message, NULL);
 			/* We need the main event loop so the user has a chance to see the dialog. */
 			nautilus_main_event_loop_register (GTK_OBJECT (dialog));
 			goto out;
@@ -623,23 +674,18 @@ nautilus_application_startup (NautilusApplication *application,
 			Nautilus_Shell_open_windows (shell, url_list, corba_startup_id, corba_geometry, browser_window, &ev);
 			CORBA_free (url_list);
 		} else if (!no_default_window) {
+			g_assert (session_to_load == NULL);
 			Nautilus_Shell_open_default_window (shell, corba_startup_id, corba_geometry, browser_window, &ev);
 		}
+
+		if (session_to_load != NULL) {
+			Nautilus_Shell_load_session (shell, session_to_load, &ev);
+		}
 		
 		/* Add ourselves to the session */
 		init_session ();
 	}
 
-	/* We're done with the shell now, so let it go. */
-	/* HACK: Don't bother releasing the shell in the case where we
-	 * just told it to quit -- that just leads to hangs and does
-	 * no good. We could probably fix this in some fancier way if
-	 * we could figure out a better lifetime rule.
-	 */
-	if (!(kill_shell || restart_shell)) {
-		bonobo_object_release_unref (shell, NULL);
-	}
-
  out:
 	CORBA_exception_free (&ev);
 }
@@ -765,6 +811,9 @@ nautilus_application_open_desktop (NautilusApplication *application)
 {
 	if (nautilus_application_desktop_windows == NULL) {
 		nautilus_application_create_desktop_windows (application);
+		
+		/* Make sure we update the session when the desktop is created */
+		update_session (gnome_master_client ());
 	}
 }
 
@@ -776,6 +825,9 @@ nautilus_application_close_desktop (void)
 				(GFunc) gtk_widget_destroy, NULL);
 		g_list_free (nautilus_application_desktop_windows);
 		nautilus_application_desktop_windows = NULL;
+
+		/* Make sure we update the session when the desktop goes away */
+		update_session (gnome_master_client ());
 	}
 }
 
@@ -984,6 +1036,8 @@ nautilus_application_present_spatial_window (NautilusApplication *application,
 									   screen);
 }
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+
 static void
 sn_error_trap_push (SnDisplay *display,
                     Display   *xdisplay)
@@ -1057,6 +1111,8 @@ end_startup_notification (GtkWidget  *widget,
 	sn_display_unref (sn_display);
 }
 
+#endif
+
 NautilusWindow *
 nautilus_application_present_spatial_window_with_selection (NautilusApplication *application,
 							    NautilusWindow      *requesting_window,
@@ -1082,21 +1138,30 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 			existing_location = existing_window->details->location;
 		}
 
-		if (eel_uris_match (existing_location, location)) {
+		if (gnome_vfs_uris_match (existing_location, location)) {
+#ifdef HAVE_STARTUP_NOTIFICATION
 			end_startup_notification (GTK_WIDGET (existing_window),
 						  startup_id);
+#endif
 
 			gtk_window_present (GTK_WINDOW (existing_window));
-			if (new_selection) {
+			if (new_selection &&
+			    existing_window->content_view != NULL) {
 				nautilus_view_set_selection (existing_window->content_view, new_selection);
 			}
+
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "present EXISTING spatial window=%p: %s",
+					    existing_window, location);
 			return existing_window;
 		}
 	}
 
 	window = create_window (application, NAUTILUS_TYPE_SPATIAL_WINDOW, startup_id, screen);
+#ifdef HAVE_STARTUP_NOTIFICATION
 	end_startup_notification (GTK_WIDGET (window),
 				  startup_id);
+#endif
 	if (requesting_window) {
 		/* Center the window over the requesting window by default */
 		int orig_x, orig_y, orig_width, orig_height;
@@ -1125,22 +1190,75 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 			   spatial_window_destroyed_callback, NULL);
 	
 	nautilus_window_go_to_with_selection (window, location, new_selection);
+
+	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "present NEW spatial window=%p: %s",
+			    window, location);
 	
 	return window;
 }
 
+static gboolean
+another_navigation_window_already_showing (NautilusWindow *the_window)
+{
+	GList *list, *item;
+	
+	list = nautilus_application_get_window_list ();
+	for (item = list; item != NULL; item = item->next) {
+		if (item->data != the_window &&
+		    NAUTILUS_IS_NAVIGATION_WINDOW (item->data)) {
+			return TRUE;
+		}
+	}
+	
+	return FALSE;
+}
+
 NautilusWindow *
 nautilus_application_create_navigation_window (NautilusApplication *application,
 					       const char          *startup_id,
 					       GdkScreen           *screen)
 {
 	NautilusWindow *window;
+	char *geometry_string;
+	gboolean maximized;
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 	
 	window = create_window (application, NAUTILUS_TYPE_NAVIGATION_WINDOW, startup_id, screen);
+#ifdef HAVE_STARTUP_NOTIFICATION
 	end_startup_notification (GTK_WIDGET (window),
 				  startup_id);
+#endif
+
+	maximized = eel_preferences_get_boolean
+			(NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_MAXIMIZED);
+	if (maximized) {
+		gtk_window_maximize (GTK_WINDOW (window));
+	} else {
+		gtk_window_unmaximize (GTK_WINDOW (window));
+	}
+
+	geometry_string = eel_preferences_get
+			(NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY);
+	if (geometry_string != NULL &&
+	    geometry_string[0] != 0) {
+		/* Ignore saved window position if a window with the same
+		 * location is already showing. That way the two windows
+		 * wont appear at the exact same location on the screen.
+		 */
+		eel_gtk_window_set_initial_geometry_from_string 
+			(GTK_WINDOW (window), 
+			 geometry_string,
+			 NAUTILUS_WINDOW_MIN_WIDTH, 
+			 NAUTILUS_WINDOW_MIN_HEIGHT,
+			 another_navigation_window_already_showing (window));
+	}
+	g_free (geometry_string);
+
+	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "create new navigation window=%p",
+			    window);
 
 	return window;
 }
@@ -1167,11 +1285,6 @@ desktop_changed_callback (gpointer user_data)
 	} else {
 		nautilus_application_close_desktop ();
 	}
-
-	/* Can't make this function just watch the preference
-	 * itself changing since ordering is important
-	 */
-	update_session (gnome_master_client ());
 }
 
 static gboolean
@@ -1268,44 +1381,282 @@ volume_unmounted_callback (GnomeVFSVolumeMonitor *monitor,
 static void
 removed_from_session (GnomeClient *client, gpointer data)
 {
-	nautilus_main_event_loop_quit ();
+	nautilus_main_event_loop_quit (FALSE);
+}
+
+static char *
+save_session_to_file (void)
+{
+	xmlDocPtr doc;
+	xmlNodePtr root_node, history_node;
+	GList *l;
+	char *dir, *filename;
+	unsigned n_processed;
+	int fd;
+	xmlSaveCtxtPtr ctx;
+
+	doc = xmlNewDoc ("1.0");
+
+	root_node = xmlNewNode (NULL, "session");
+	xmlDocSetRootElement (doc, root_node);
+
+	history_node = xmlNewChild (root_node, NULL, "history", NULL);
+
+	n_processed = 0;
+	for (l = nautilus_get_history_list (); l != NULL; l = l->next) {
+		NautilusBookmark *bookmark;
+		xmlNodePtr bookmark_node;
+		char *tmp;
+
+		bookmark = l->data;
+
+		bookmark_node = xmlNewChild (history_node, NULL, "bookmark", NULL);
+
+		tmp = nautilus_bookmark_get_name (bookmark);
+		xmlNewProp (bookmark_node, "name", tmp);
+		g_free (tmp);
+
+		tmp = nautilus_bookmark_get_icon (bookmark);
+		xmlNewProp (bookmark_node, "icon", tmp);
+		g_free (tmp);
+
+		tmp = nautilus_bookmark_get_uri (bookmark);
+		xmlNewProp (bookmark_node, "uri", tmp);
+		g_free (tmp);
+
+		if (nautilus_bookmark_get_has_custom_name (bookmark)) {
+			xmlNewProp (bookmark_node, "has_custom_name", "TRUE");
+		}
+
+		if (++n_processed > 50) { /* prevent history list from growing arbitrarily large. */
+			break;
+		}
+	}
+
+	for (l = nautilus_application_window_list; l != NULL; l = l->next) {
+		xmlNodePtr win_node;
+		NautilusWindow *window;
+		char *tmp;
+
+		window = l->data;
+
+		win_node = xmlNewChild (root_node, NULL, "window", NULL);
+
+		xmlNewProp (win_node, "type", NAUTILUS_IS_NAVIGATION_WINDOW (window) ? "navigation" : "spatial");
+
+		if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) { /* spatial windows store their state as file metadata */
+			tmp = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
+			xmlNewProp (win_node, "geometry", tmp);
+			g_free (tmp);
+
+			if (GTK_WIDGET (window)->window &&
+			    gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_MAXIMIZED) {
+				xmlNewProp (win_node, "maximized", "TRUE");
+			}
+		}
+
+		tmp = nautilus_window_get_location (window);
+		xmlNewProp (win_node, "location", tmp);
+		g_free (tmp);
+	}
+
+	dir = nautilus_get_user_directory ();
+	filename = g_build_filename (dir, "saved-session-XXXXXX", NULL);
+	g_free (dir);
+
+	fd = g_mkstemp (filename);
+	if (fd < 0) {
+		g_message ("failed to open session file %s", filename);
+		g_free (filename);
+		filename = NULL;
+		goto out;
+	}
+
+	xmlIndentTreeOutput = 1;
+	ctx = xmlSaveToFd (fd, NULL, XML_SAVE_FORMAT);
+	if (xmlSaveDoc (ctx, doc) < 0 ||
+	    xmlSaveFlush (ctx) < 0) {
+		g_message ("failed to save session to %s", filename);
+		g_free (filename);
+		filename = NULL;
+	}
+	
+	xmlSaveClose(ctx);
+	close (fd);	
+
+ out:
+	xmlFreeDoc (doc);
+
+	return filename;
+}
+
+void
+nautilus_application_load_session (NautilusApplication *application,
+				   const char *filename)
+{
+	xmlDocPtr doc;
+	gboolean bail;
+
+	g_assert (filename != NULL);
+
+	bail = TRUE;
+
+	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+		xmlNodePtr root_node;
+
+		doc = xmlReadFile (filename, NULL, 0);
+		if (doc != NULL && (root_node = xmlDocGetRootElement (doc)) != NULL) {
+			xmlNodePtr node;
+
+			bail = FALSE;
+
+			for (node = root_node->children; node != NULL; node = node->next) {
+
+				if (!strcmp (node->name, "text")) {
+					continue;
+				} else if (!strcmp (node->name, "history")) {
+					xmlNodePtr bookmark_node;
+					gboolean emit_change;
+
+					emit_change = FALSE;
+
+					for (bookmark_node = node->children; bookmark_node != NULL; bookmark_node = bookmark_node->next) {
+						if (!strcmp (bookmark_node->name, "text")) {
+							continue;
+						} else if (!strcmp (bookmark_node->name, "bookmark")) {
+							xmlChar *name, *icon, *uri;
+							gboolean has_custom_name;
+
+							uri = xmlGetProp (bookmark_node, "uri");
+							name = xmlGetProp (bookmark_node, "name");
+							has_custom_name = xmlHasProp (bookmark_node, "has_custom_name") ? TRUE : FALSE;
+							icon = xmlGetProp (bookmark_node, "icon");
+
+							emit_change |= nautilus_add_to_history_list_no_notify (uri, name, has_custom_name, icon);
+
+							xmlFree (name);
+							xmlFree (uri);
+							xmlFree (icon);
+						} else {
+							g_message ("unexpected bookmark node %s while parsing %s", bookmark_node->name, filename);
+							bail = TRUE;
+							continue;
+						}
+					}
+
+					if (emit_change) {
+						nautilus_send_history_list_changed ();
+					}
+				} else if (!strcmp (node->name, "window")) {
+					NautilusWindow *window;
+					xmlChar *type, *location;
+
+					type = xmlGetProp (node, "type");
+					if (type == NULL) {
+						g_message ("empty type node while parsing %s", filename);
+						bail = TRUE;
+						continue;
+					}
+
+					location = xmlGetProp (node, "location");
+					if (location == NULL) {
+						g_message ("empty location node while parsing %s", filename);
+						bail = TRUE;
+						xmlFree (type);
+						continue;
+					}
+
+					if (!strcmp (type, "navigation")) {
+						xmlChar *geometry;
+
+						window = nautilus_application_create_navigation_window (application, NULL, gdk_screen_get_default ());
+
+						geometry = xmlGetProp (node, "geometry");
+						if (geometry != NULL) {
+							eel_gtk_window_set_initial_geometry_from_string 
+								(GTK_WINDOW (window), 
+								 geometry,
+								 NAUTILUS_WINDOW_MIN_WIDTH, 
+								 NAUTILUS_WINDOW_MIN_HEIGHT,
+								 FALSE);
+						}
+						xmlFree (geometry);
+
+						if (xmlHasProp (node, "maximized")) {
+							gtk_window_maximize (GTK_WINDOW (window));
+						} else {
+							gtk_window_unmaximize (GTK_WINDOW (window));
+						}
+
+						nautilus_window_open_location (window, location, FALSE);
+					} else if (!strcmp (type, "spatial")) {
+						window = nautilus_application_present_spatial_window (application, NULL, NULL, location, gdk_screen_get_default ());
+					} else {
+						g_message ("unknown window type \"%s\" while parsing %s", type, filename);
+						bail = TRUE;
+					}
+
+					xmlFree (type);
+					xmlFree (location);
+				} else {
+					g_message ("unexpected node %s while parsing %s", node->name, filename);
+					bail = TRUE;
+					continue;
+				}
+			}
+		}
+
+		if (doc != NULL) {
+			xmlFreeDoc (doc);
+		}
+	}
+
+	if (bail) {
+		g_message ("failed to load session from %s", filename);
+	} else {
+		char *uri;
+		GnomeVFSFileInfo *info;
+		/* only remove file if it is regular, user-owned and the user has write access. */
+
+		uri = gnome_vfs_get_uri_from_local_path (filename);
+		info = gnome_vfs_file_info_new ();
+		if (uri != NULL &&
+		    gnome_vfs_get_file_info (uri, info, GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK &&
+		    (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE) &&
+		    (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS) &&
+		    (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_IDS) &&
+		    info->type == GNOME_VFS_FILE_TYPE_REGULAR &&
+		    (info->permissions & (GNOME_VFS_PERM_USER_WRITE |
+					  GNOME_VFS_PERM_USER_WRITE)) &&
+		    info->uid == geteuid ()) {
+			g_remove (filename);
+		}
+		gnome_vfs_file_info_unref (info);
+		g_free (uri);
+	}
 }
 
 static gint
 save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style, gint shutdown,
 	      GnomeInteractStyle interact_style, gint fast, gpointer data)
 {
-	NautilusWindow *window;
-	GList *l;
-	static char *clone_argv[] = { "nautilus", "--no-default-window" };
-	char **restart_argv;
-	int argc;
-	int i;
-	int num_windows;
+	char *argv[3] = { NULL };
 
-	num_windows = g_list_length (nautilus_application_window_list);
-	if (num_windows > 0) {
-		argc = 1 + num_windows;
-		i = 0;
-		restart_argv = g_new (char *, argc);
-		restart_argv[i++] = g_strdup ("nautilus");
-		for (l = nautilus_application_window_list; l != NULL; l = l->next) {
-			window = NAUTILUS_WINDOW (l->data);
-			restart_argv[i++] = nautilus_window_get_location (window);
-		}
-		
-		gnome_client_set_restart_command (client, argc, restart_argv);
+	argv[0] = "nautilus";
 
-		for (i = 0; i < argc; i++) {
-			g_free (restart_argv[i]);
-		}
-		g_free (restart_argv);
-	} else {
-		gnome_client_set_restart_command (client, 
-						  G_N_ELEMENTS (clone_argv), 
-						  clone_argv);
+	argv[2] = save_session_to_file ();
+	if (argv[2] != NULL) {
+		argv[1] = "--load-session";
 	}
-	
+
+	gnome_client_set_restart_command (client, 
+					  G_N_ELEMENTS (argv), 
+					  argv);
+
+	if (argv[2] != NULL) {
+		g_free (argv[2]);
+	}
+
 	return TRUE;
 }
 
