@@ -51,6 +51,7 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkaboutdialog.h>
 #include <gtk/gtkenums.h>
+#include <gtk/gtkversion.h>
 #include <libgnome/gnome-help.h>
 #include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
@@ -433,25 +434,22 @@ action_about_nautilus_callback (GtkAction *action,
 		   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA")
 	};
 	gchar *license_trans;
-	gchar *logo_path;
-	GdkPixbuf *logo = NULL;
 
 	license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]),
 					     _(license[2]), NULL);
 
-	logo_path = nautilus_pixmap_file ("nautilus-launch-icon.png");
-	if (logo_path != NULL) {
-		logo = gdk_pixbuf_new_from_file (logo_path, NULL);
-	}
-	
 	gtk_show_about_dialog (GTK_WINDOW (user_data),
+#if GTK_CHECK_VERSION (2, 11, 0)
+			       "program-name", _("Nautilus"),
+#else
 			       "name", _("Nautilus"),
+#endif /* GTK 2.11. 0 */
 			       "version", VERSION,
 			       "comments", _("Nautilus is a graphical shell "
 					     "for GNOME that makes it "
 					     "easy to manage your files "
 					     "and the rest of your system."),
-			       "copyright", _("Copyright \xC2\xA9 1999-2006 "
+			       "copyright", _("Copyright \xC2\xA9 1999-2007 "
 					      "The Nautilus authors"),
 			       "license", license_trans,
 			       "wrap-license", TRUE,
@@ -462,15 +460,13 @@ action_about_nautilus_callback (GtkAction *action,
 				 * box to give credit to the translator(s).
 				 */
 			      "translator-credits", _("translator-credits"),
-			      "logo", logo,
+			      "logo-icon-name", "nautilus",
+			      "website", "http://www.gnome.org/projects/nautilus",
+			      "website-label", _("Nautilus Web Site"),
 			      NULL);
 
-	if (logo != NULL) {
-		g_object_unref (logo);
-	}
-
 	g_free (license_trans);
-	g_free (logo_path);
+
 }
 
 static void
@@ -661,31 +657,31 @@ static const GtkActionEntry main_entries[] = {
     G_CALLBACK (action_zoom_normal_callback) },
   { "Connect to Server", NULL,                        /* name, stock id */
     N_("Connect to _Server..."), NULL,           /* label, accelerator */
-    N_("Set up a connection to a network server"),                                      /* tooltip */ 
+    N_("Connect to a remote computer or shared disk"),                                      /* tooltip */ 
     G_CALLBACK (action_connect_to_server_callback) },
   { "Home", "gnome-fs-home",                        /* name, stock id */
-    N_("_Home"), "<alt>Home",           /* label, accelerator */
-    N_("Go to the home folder"),                                  /* tooltip */ 
+    N_("_Home Folder"), "<alt>Home",           /* label, accelerator */
+    N_("Open your personal folder"),                                  /* tooltip */ 
     G_CALLBACK (action_home_callback) },
   { "Go to Computer", "gnome-fs-client",                        /* name, stock id */
     N_("_Computer"), NULL,           /* label, accelerator */
-    N_("Go to the computer location"),                                  /* tooltip */ 
+    N_("Browse all local and remote disks and folders accessible from this computer"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_computer_callback) },
   { "Go to Network", "gnome-fs-network",                        /* name, stock id */
     N_("_Network"), NULL,           /* label, accelerator */
-    N_("Go to the network location"),                                  /* tooltip */ 
+    N_("Browse bookmarked and local network locations"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_network_callback) },
   { "Go to Templates", "stock_new-template",                        /* name, stock id */
     N_("T_emplates"), NULL,           /* label, accelerator */
-    N_("Go to the templates folder"),                                  /* tooltip */ 
+    N_("Open your personal templates folder"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_templates_callback) },
-  { "Go to Trash", "gnome-fs-trash-empty",                        /* name, stock id */
+  { "Go to Trash", "user-trash",                        /* name, stock id */
     N_("_Trash"), NULL,           /* label, accelerator */
-    N_("Go to the trash folder"),                                  /* tooltip */ 
+    N_("Open your personal trash folder"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_trash_callback) },
   { "Go to Burn CD", "nautilus-cd-burner",                        /* name, stock id */
     N_("CD/_DVD Creator"), NULL,           /* label, accelerator */
-    N_("Go to the CD/DVD Creator"),                                  /* tooltip */ 
+    N_("Open a folder into which you can drag files to burn to a CD or DVD"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_burn_cd_callback) },
 };
 
@@ -712,7 +708,10 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 
 	action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_UP);
 	g_object_set (action, "short_label", _("_Up"), NULL);
-	
+
+	action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_HOME);
+	g_object_set (action, "short_label", _("_Home"), NULL);
+
 	window->details->ui_manager = gtk_ui_manager_new ();
 	ui_manager = window->details->ui_manager;
 	gtk_window_add_accel_group (GTK_WINDOW (window),
